# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

try:
    from trytond.modules.sale_shipment_grouping.tests.test_sale_shipment_grouping import suite  # noqa: E501
except ImportError:
    from .test_sale_shipment_grouping import suite

__all__ = ['suite']



# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
from trytond.model import fields
from trytond.pool import Pool, PoolMeta

sale_shipment_grouping_method = fields.Selection(
    'get_sale_shipment_grouping_methods', "Sale Shipment Grouping Method",
    help="The default shipment grouping method for new customers.")


@classmethod
def get_sale_shipment_grouping_methods(cls):
    pool = Pool()
    Party = pool.get('party.party')
    field_name = 'sale_shipment_grouping_method'
    return Party.fields_get([field_name])[field_name]['selection']


class Configuration(metaclass=PoolMeta):
    __name__ = 'sale.configuration'

    sale_shipment_grouping_method = fields.MultiValue(
        sale_shipment_grouping_method)
    get_sale_shipment_grouping_methods = get_sale_shipment_grouping_methods

    @classmethod
    def multivalue_model(cls, field):
        pool = Pool()
        if field == 'sale_shipment_grouping_method':
            return pool.get('sale.configuration.sale_method')
        return super(Configuration, cls).multivalue_model(field)


class ConfigurationSaleMethod(metaclass=PoolMeta):
    __name__ = 'sale.configuration.sale_method'

    sale_shipment_grouping_method = sale_shipment_grouping_method
    get_sale_shipment_grouping_methods = get_sale_shipment_grouping_methods



# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

from trytond.pool import Pool
from . import party
from . import sale
from . import configuration


def register():
    Pool.register(
        party.Party,
        party.PartySaleShipmentGroupingMethod,
        sale.Sale,
        configuration.Configuration,
        configuration.ConfigurationSaleMethod,
        module='sale_shipment_grouping', type_='model')



# This file is part of Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

from trytond.pool import PoolMeta
from trytond.transaction import Transaction


class Sale(metaclass=PoolMeta):
    __name__ = 'sale.sale'

    @property
    def shipment_grouping_method(self):
        return self.party.sale_shipment_grouping_method

    @property
    def _shipment_grouping_fields(self):
        return ('customer', 'delivery_address', 'company', 'warehouse')

    def _get_grouped_shipment_planned_date(self, shipment):
        return [('planned_date', '=', shipment.planned_date)]

    def _get_grouped_shipment_order(self):
        "Returns the order used to find shipments that should be grouped"
        return None

    def _get_grouped_shipment_domain(self, shipment):
        "Returns a domain that will find shipments that should be grouped"
        Shipment = shipment.__class__
        shipment_domain = [
            ('moves.origin', 'like', 'sale.line,%'),
            ('state', 'in', ['draft', 'waiting']),
            ]
        shipment_domain += self._get_grouped_shipment_planned_date(shipment)
        fields = [
            f for f in self._shipment_grouping_fields if f in Shipment._fields]
        defaults = Shipment.default_get(fields, with_rec_name=False)
        for field in fields:
            shipment_domain.append((field, '=',
                    getattr(shipment, field, defaults.get(field))))
        return shipment_domain

    def _get_shipment_sale(self, shipment_type, values):
        transaction = Transaction()
        context = transaction.context
        shipment = super(Sale, self)._get_shipment_sale(shipment_type, values)
        if (not context.get('skip_grouping', False)
                and self.shipment_grouping_method):
            with transaction.set_context(skip_grouping=True):
                shipment = self._get_shipment_sale(shipment_type, values)
            Shipment = shipment.__class__
            domain = self._get_grouped_shipment_domain(shipment)
            order = self._get_grouped_shipment_order()
            grouped_shipments = Shipment.search(domain, order=order, limit=1)
            if grouped_shipments:
                shipment, = grouped_shipments
        return shipment



# This file is part of Tryton.  The COPYRIGHT file at the top level of this
# repository contains the full copyright notices and license terms.
from trytond import backend
from trytond.model import ModelSQL, ValueMixin, fields
from trytond.pool import PoolMeta, Pool
from trytond.tools.multivalue import migrate_property


class Party(metaclass=PoolMeta):
    __name__ = 'party.party'

    sale_shipment_grouping_method = fields.MultiValue(fields.Selection([
                (None, 'None'),
                ('standard', 'Standard'),
                ],
            'Sale Shipment Grouping Method'))
    sale_shipment_grouping_methods = fields.One2Many(
        'party.party.sale_shipment_grouping_method', 'party',
        "Sale Shipment Grouping Methods")

    @classmethod
    def default_sale_shipment_grouping_method(cls, **pattern):
        pool = Pool()
        Configuration = pool.get('sale.configuration')
        return Configuration(1).get_multivalue(
            'sale_shipment_grouping_method', **pattern)


class PartySaleShipmentGroupingMethod(ModelSQL, ValueMixin):
    "Party Sale Shipment Grouping Method"
    __name__ = 'party.party.sale_shipment_grouping_method'
    party = fields.Many2One(
        'party.party', "Party", ondelete='CASCADE', select=True)
    sale_shipment_grouping_method = fields.Selection(
        'get_sale_shipment_grouping_methods', "Sale Shipment Grouping Method")

    @classmethod
    def __register__(cls, module_name):
        exist = backend.TableHandler.table_exist(cls._table)

        super(PartySaleShipmentGroupingMethod, cls).__register__(module_name)

        if not exist:
            cls._migrate_property([], [], [])

    @classmethod
    def _migrate_property(cls, field_names, value_names, fields):
        field_names.append('sale_shipment_grouping_method')
        value_names.append('sale_shipment_grouping_method')
        migrate_property(
            'party.party', field_names, cls, value_names,
            parent='party', fields=fields)

    @classmethod
    def get_sale_shipment_grouping_methods(cls):
        pool = Pool()
        Party = pool.get('party.party')
        field_name = 'sale_shipment_grouping_method'
        return Party.fields_get([field_name])[field_name]['selection']
